@@ -15,14 +15,18 @@
  */
 package org.pf4j;
 
+import com.google.testing.compile.JavaFileObjects;
 import org.junit.jupiter.api.Test;
-import org.pf4j.test.AnotherFailTestPlugin;
-import org.pf4j.test.AnotherTestPlugin;
-import org.pf4j.test.FailTestPlugin;
+import org.pf4j.test.JavaFileObjectClassLoader;
+import org.pf4j.test.JavaFileObjectUtils;
+import org.pf4j.test.JavaSources;
 import org.pf4j.test.TestPlugin;
 
+import javax.tools.JavaFileObject;
+
 import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.mockito.Mockito.mock;
@@ -33,6 +37,29 @@ import static org.mockito.Mockito.when;
  */
 public class DefaultPluginFactoryTest {
 
+    public static final JavaFileObject FailTestPlugin = JavaFileObjects.forSourceLines("FailTestPlugin",
+        "package test;",
+        "import org.pf4j.Plugin;",
+        "",
+        "public class FailTestPlugin {",
+        "}");
+
+    public static final JavaFileObject AnotherFailTestPlugin = JavaFileObjects.forSourceLines("AnotherFailTestPlugin",
+        "package test;",
+        "import org.pf4j.Plugin;",
+        "",
+        "public class AnotherFailTestPlugin extends Plugin {",
+        "     public AnotherFailTestPlugin() { super(null); }",
+        "}");
+
+    public static final JavaFileObject AnotherTestPlugin = JavaFileObjects.forSourceLines("AnotherTestPlugin",
+        "package test;",
+        "import org.pf4j.Plugin;",
+        "",
+        "public class AnotherTestPlugin extends Plugin {",
+        "     public AnotherTestPlugin() { super(); }",
+        "}");
+
     @Test
     public void testCreate() {
         PluginDescriptor pluginDescriptor = mock(PluginDescriptor.class);
@@ -52,27 +79,35 @@ public class DefaultPluginFactoryTest {
     @Test
     public void pluginConstructorNoParameters() {
         PluginDescriptor pluginDescriptor = mock(PluginDescriptor.class);
-        when(pluginDescriptor.getPluginClass()).thenReturn(AnotherTestPlugin.class.getName());
+        JavaFileObject object = JavaSources.compile(AnotherTestPlugin);
+        String pluginClassName = JavaFileObjectUtils.getClassName(object);
+        when(pluginDescriptor.getPluginClass()).thenReturn(pluginClassName);
 
         PluginWrapper pluginWrapper = mock(PluginWrapper.class);
         when(pluginWrapper.getDescriptor()).thenReturn(pluginDescriptor);
-        when(pluginWrapper.getPluginClassLoader()).thenReturn(getClass().getClassLoader());
+        JavaFileObjectClassLoader classLoader = new JavaFileObjectClassLoader();
+        classLoader.load(AnotherTestPlugin);
+        when(pluginWrapper.getPluginClassLoader()).thenReturn(classLoader);
 
         PluginFactory pluginFactory = new DefaultPluginFactory();
 
         Plugin result = pluginFactory.create(pluginWrapper);
         assertNotNull(result);
-        assertThat(result, instanceOf(AnotherTestPlugin.class));
+        assertEquals(pluginClassName, result.getClass().getName());
     }
 
     @Test
     public void testCreateFail() {
         PluginDescriptor pluginDescriptor = mock(PluginDescriptor.class);
-        when(pluginDescriptor.getPluginClass()).thenReturn(FailTestPlugin.class.getName());
+        JavaFileObject object = JavaSources.compile(FailTestPlugin);
+        String pluginClassName = JavaFileObjectUtils.getClassName(object);
+        when(pluginDescriptor.getPluginClass()).thenReturn(pluginClassName);
 
         PluginWrapper pluginWrapper = mock(PluginWrapper.class);
         when(pluginWrapper.getDescriptor()).thenReturn(pluginDescriptor);
-        when(pluginWrapper.getPluginClassLoader()).thenReturn(getClass().getClassLoader());
+        JavaFileObjectClassLoader classLoader = new JavaFileObjectClassLoader();
+        classLoader.load(FailTestPlugin);
+        when(pluginWrapper.getPluginClassLoader()).thenReturn(classLoader);
 
         PluginFactory pluginFactory = new DefaultPluginFactory();
 
@@ -98,11 +133,15 @@ public class DefaultPluginFactoryTest {
     @Test
     public void testCreateFailConstructor() {
         PluginDescriptor pluginDescriptor = mock(PluginDescriptor.class);
-        when(pluginDescriptor.getPluginClass()).thenReturn(AnotherFailTestPlugin.class.getName());
+        JavaFileObject object = JavaSources.compile(AnotherFailTestPlugin);
+        String pluginClassName = JavaFileObjectUtils.getClassName(object);
+        when(pluginDescriptor.getPluginClass()).thenReturn(pluginClassName);
 
         PluginWrapper pluginWrapper = mock(PluginWrapper.class);
         when(pluginWrapper.getDescriptor()).thenReturn(pluginDescriptor);
-        when(pluginWrapper.getPluginClassLoader()).thenReturn(getClass().getClassLoader());
+        JavaFileObjectClassLoader classLoader = new JavaFileObjectClassLoader();
+        classLoader.load(AnotherFailTestPlugin);
+        when(pluginWrapper.getPluginClassLoader()).thenReturn(classLoader);
 
         PluginFactory pluginFactory = new DefaultPluginFactory();
 
